@@ -35,7 +35,7 @@ other contours. \n\
 \n\
 The initialisation arguments may be:\n\
 - another Polygon instance which will be cloned, or\n\
-- a string or file object which will be read, or\n\
+- the name of a file which will be read, or\n\
 - a sequence of points which will be used as the first contour.\n\
 A point is a sequence of two floats.\n\
 \n\
@@ -77,7 +77,7 @@ Operations on polygons:\n\
 #define INDEF INT_MAX
 
 #ifndef POLY_VERSION
-#define POLY_VERSION "2.0.8"
+#define POLY_VERSION "3.0.8"
 #endif
 
 #ifndef PyMODINIT_FUNC
@@ -115,10 +115,10 @@ typedef struct {
     int bbValid;
 } Polygon;
 
-staticforward PyTypeObject    Polygon_Type;
-staticforward PyMethodDef     Polygon_Methods[26];
-staticforward PyNumberMethods Polygon_NumberMethods;
-staticforward PySequenceMethods Polygon_SequenceMethods;
+static PyTypeObject    Polygon_Type;
+static PyMethodDef     Polygon_Methods[26];
+static PyNumberMethods Polygon_NumberMethods;
+static PySequenceMethods Polygon_SequenceMethods;
 
 /* check if object o is a Polygon */
 #define Polygon_Check(o) (PyObject_TypeCheck(o, &Polygon_Type))
@@ -147,7 +147,7 @@ static void Polygon_dealloc(Polygon *self) {
     gpc_free_polygon(self->gpc_p);
     free(self->gpc_p);
     Py_XDECREF(self->attr);
-    self->ob_type->tp_free((PyObject*)self);
+    self->ob_base.ob_type->tp_free((PyObject*)self);
 }
 
 
@@ -157,7 +157,7 @@ static void Polygon_dealloc(Polygon *self) {
 Writes Polygon data to a file in gpc format.\n\
 \n\
 :Arguments:\n\
-    - file: writable file object or filename string\n\
+    - file: file name string\n\
     - optional holeflag: bool\n\
 :Returns:\n\
     None\n"
@@ -166,10 +166,8 @@ static PyObject *Polygon_write(Polygon *self, PyObject *args) {
     int hflag = 1;
     if (! PyArg_ParseTuple(args, "O|i", &O, &hflag))
         return Polygon_Raise(ERR_ARG);
-    if (PyFile_Check(O))
-        gpc_write_polygon(PyFile_AsFile(O), hflag, self->gpc_p);
-    else if (PyString_Check(O)) {
-        FILE *f = fopen(PyString_AsString(O), "w");
+    if (PyUnicode_Check(O)) {
+        FILE *f = fopen(PyBytes_AsString(PyUnicode_AsEncodedString(O, Py_FileSystemDefaultEncoding, NULL)), "w");
         if (!f)
             return Polygon_Raise(PyExc_IOError, "Could not open file for writing!");
         gpc_write_polygon(f, hflag, self->gpc_p);
@@ -184,7 +182,7 @@ static PyObject *Polygon_write(Polygon *self, PyObject *args) {
 "read(file)\n\
 Reads Polygon data from a file in gpc format.\n\
 :Arguments:\n\
-    - file: readable file object or filename string\n\
+    - file: file name string\n\
     - optional holeflag: bool\n\
 :Returns:\n\
     None\n\
@@ -194,10 +192,8 @@ static PyObject *Polygon_read(Polygon *self, PyObject *args) {
     int hflag = 1;
     if (! PyArg_ParseTuple(args, "O|i", &O, &hflag))
         return Polygon_Raise(ERR_ARG);
-    if (PyFile_Check(O))
-        gpc_read_polygon(PyFile_AsFile(O), hflag, self->gpc_p);
-    else if (PyString_Check(O)) {
-        FILE *f = fopen(PyString_AsString(O), "r");
+    if (PyUnicode_Check(O)) {
+        FILE *f = fopen(PyBytes_AsString(PyUnicode_AsEncodedString(O, Py_FileSystemDefaultEncoding, NULL)), "r");
         if (!f)
             return Polygon_Raise(PyExc_IOError, "Could not open file for reading!");
         gpc_read_polygon(f, hflag, self->gpc_p);
@@ -311,12 +307,10 @@ static int Polygon_init(Polygon *self, PyObject *args, PyObject *kwds) {
                 Polygon_Raise(ERR_MEM);
                 return -1;
             }
-        } else if (PyString_Check(O)) {
+        } else if (PyUnicode_Check(O)) {
             TMP = Polygon_read(self, args);
         } else if (PySequence_Check(O)) {
             TMP = Polygon_addContour(self, args);
-        } else if (PyFile_Check(O)) {
-            TMP = Polygon_read(self, args);
         } else {
             Polygon_Raise(ERR_ARG);
             return -1;
@@ -338,24 +332,24 @@ static PyObject *Polygon_repr(PyObject *self) {
     gpc_polygon * p = ((Polygon *)self)->gpc_p;
     gpc_vertex_list *vl;
     gpc_vertex *v;
-    PyObject * s = PyString_FromString("Polygon:");
+    PyObject * s = PyBytes_FromString("Polygon:");
     vl = p->contour;
     for(i=0; i<p->num_contours; i++) {
         if (*((p->hole)+i) == 0) 
             sprintf(buf, "\n  <%d:Contour:", i);
         else
             sprintf(buf, "\n  <%d:Hole   :", i);
-        PyString_ConcatAndDel(&s, PyString_FromString(buf));
+        PyBytes_ConcatAndDel(&s, PyBytes_FromString(buf));
         v = vl->vertex;
         for (j = 0; j < vl->num_vertices; j++) {
             sprintf(buf, " [%d:%#.2g, %#.2g]", j, v->x, v->y);
-            PyString_ConcatAndDel(&s, PyString_FromString(buf));
+            PyBytes_ConcatAndDel(&s, PyBytes_FromString(buf));
             v++;
         }
-        PyString_ConcatAndDel(&s, PyString_FromString(">"));
+        PyBytes_ConcatAndDel(&s, PyBytes_FromString(">"));
         vl++;
     }
-    return s;
+    return PyUnicode_FromEncodedObject(s, "ascii", "strict");
 }
 
 
@@ -1065,7 +1059,6 @@ static PyObject *Polygon_triStrip(Polygon *self) {
 }
 
 
-
 /* sample() was contributed by Thouis (Ray) Jones */
 #define DOCSTR_POLY_SAMPLE \
 "sample(rng)\n\
@@ -1098,7 +1091,7 @@ static PyObject *Polygon_sample(Polygon *self, PyObject *args) {
         Polygon_Raise(PolyError, "rng raised an error");
         goto cleanup;
     }
-    
+
     if ((! PyFloat_Check(val1)) || (! PyFloat_Check(val2)) || (! PyFloat_Check(val3))) {
          Polygon_Raise(PolyError,
                        "rng returned something other than a float");
@@ -1161,36 +1154,6 @@ static PyObject *Polygon_sample(Polygon *self, PyObject *args) {
 }
 
 
-static PyObject *Polygon_getattr(Polygon *self, char *name) {
-    if (self->attr != NULL) {
-        PyObject *v = PyDict_GetItemString(self->attr, name);
-        if (v != NULL) {
-            Py_INCREF(v);
-            return v;
-        }
-    }
-    return Py_FindMethod(Polygon_Methods, (PyObject *)self, name);
-}
-
-
-static int Polygon_setattr(Polygon *self, char *name, PyObject *v) {
-    if (self->attr == NULL) {
-        self->attr = PyDict_New();
-        if (self->attr == NULL)
-            return -1;
-    }
-    if (v == NULL) {
-        int rv = PyDict_DelItemString(self->attr, name);
-        if (rv < 0)
-            PyErr_SetString(PyExc_AttributeError,
-                            "delete non-existing Polygon attribute");
-        return rv;
-    }
-    else
-        return PyDict_SetItemString(self->attr, name, v);
-}
-
-
 static int Polygon_nonzero(PyObject *self) {
     return ((((Polygon *)self)->gpc_p->num_contours > 0) ? 1 : 0);
 }
@@ -1236,7 +1199,6 @@ static PyNumberMethods Polygon_NumberMethods = {
     (PyCFunction)Polygon_opUnion, /* binaryfunc nb_add;        __add__ */
     (PyCFunction)Polygon_opDiff,  /* binaryfunc nb_subtract;   __sub__ */
     0,               /* binaryfunc nb_multiply;   __mul__ */
-    0,               /* binaryfunc nb_divide;     __div__ */
     0,               /* binaryfunc nb_remainder;  __mod__ */
     0,               /* binaryfunc nb_divmod;     __divmod__ */
     0,               /* ternaryfunc nb_power;     __pow__ */
@@ -1301,16 +1263,15 @@ static PyMethodDef Polygon_Methods[] = {
 
 
 static PyTypeObject Polygon_Type = {
-    PyObject_HEAD_INIT(NULL)
-    0,                               /*ob_size*/
+    PyVarObject_HEAD_INIT(NULL, 0)
     "cPolygon.Polygon",             /*tp_name*/
     sizeof(Polygon),                 /*tp_basicsize*/
     0,                               /*tp_itemsize*/
     /* methods */
     (destructor)Polygon_dealloc,     /*tp_dealloc*/
     0,                               /*tp_print*/
-    (getattrfunc)Polygon_getattr,    /*tp_getattr*/
-    (setattrfunc)Polygon_setattr,    /*tp_setattr*/
+    0,                               /*tp_getattr*/
+    0,                               /*tp_setattr*/
     0,                               /*tp_compare*/
     Polygon_repr,                    /* (reprfunc)tp_repr*/
     &Polygon_NumberMethods,          /*tp_as_number*/
@@ -1355,9 +1316,9 @@ with support for numpy).\n\
 :Returns:\n\
     None\n"
 static PyObject * setDataStyle(PyObject *self, PyObject *arg) {
-    if (! PyInt_Check(arg))
+    if (! PyLong_Check(arg))
         return Polygon_Raise(ERR_ARG);
-    dataStyle = PyInt_AsLong(arg);
+    dataStyle = PyLong_AsLong(arg);
     Py_RETURN_NONE;
 }
 
@@ -1373,8 +1334,6 @@ the value will be needed when point coordinates are not accurate\n\
 static PyObject * setEpsilon(PyObject *self, PyObject *arg) {
     if (PyFloat_Check(arg))
         GPC_EPSILON = PyFloat_AsDouble(arg);
-    else if (PyInt_Check(arg))
-        GPC_EPSILON = PyInt_AsLong(arg);
     else if (PyLong_Check(arg))
         GPC_EPSILON = PyLong_AsLong(arg);
     else
@@ -1400,10 +1359,20 @@ static PyMethodDef cPolygonMethods[] = {
     {NULL,      NULL}        /* Sentinel */
 };
 
-PyMODINIT_FUNC initcPolygon(void) {
+static struct PyModuleDef cPolygonModule = {
+   PyModuleDef_HEAD_INIT,
+   "cPolygon",
+   DOCSTR_POLY_MODULE,
+   -1,
+   cPolygonMethods
+};
+
+PyMODINIT_FUNC PyInit_cPolygon(void) {
     PyObject *m;
-    if (PyType_Ready(&Polygon_Type) < 0) return;
-    m = Py_InitModule3("cPolygon", cPolygonMethods, DOCSTR_POLY_MODULE);
+    if (PyType_Ready(&Polygon_Type) < 0) return NULL;
+    m = PyModule_Create(&cPolygonModule);
+    if (m == NULL)
+        return NULL;
 #ifdef WITH_NUMPY
     import_array();
 #endif /* WITH_NUMPY */
@@ -1412,15 +1381,16 @@ PyMODINIT_FUNC initcPolygon(void) {
     PyModule_AddObject(m, "Error", PolyError);
     Py_INCREF(&Polygon_Type);
     PyModule_AddObject(m, "Polygon", (PyObject *)(&Polygon_Type));
-    PyModule_AddObject(m, "STYLE_TUPLE", (PyObject *)PyInt_FromLong(STYLE_TUPLE));
-    PyModule_AddObject(m, "STYLE_LIST", (PyObject *)PyInt_FromLong(STYLE_LIST));
+    PyModule_AddObject(m, "STYLE_TUPLE", (PyObject *)PyLong_FromLong(STYLE_TUPLE));
+    PyModule_AddObject(m, "STYLE_LIST", (PyObject *)PyLong_FromLong(STYLE_LIST));
 #ifdef WITH_NUMPY
-    PyModule_AddObject(m, "STYLE_NUMPY", (PyObject *)PyInt_FromLong(STYLE_NUMPY));
+    PyModule_AddObject(m, "STYLE_NUMPY", (PyObject *)PyLong_FromLong(STYLE_NUMPY));
     PyModule_AddObject(m, "withNumPy", (PyObject *)Py_True);
 #else
     PyModule_AddObject(m, "withNumPy", (PyObject *)Py_False);
 #endif /* WITH_NUMPY */
-    PyModule_AddObject(m, "version", (PyObject *)PyString_FromString(POLY_VERSION));
-    PyModule_AddObject(m, "author", (PyObject *)PyString_FromString(POLY_AUTHOR));
-    PyModule_AddObject(m, "license", (PyObject *)PyString_FromString(POLY_LICENSE));
+    PyModule_AddObject(m, "version", (PyObject *)PyUnicode_FromString(POLY_VERSION));
+    PyModule_AddObject(m, "author", (PyObject *)PyUnicode_FromString(POLY_AUTHOR));
+    PyModule_AddObject(m, "license", (PyObject *)PyUnicode_FromString(POLY_LICENSE));
+    return m;
 }
